package info.textgrid.lab.collatex.model;

import info.textgrid.lab.collatex.model.equivalences.Equivalences;
import info.textgrid.lab.collatex.model.text.BasicWitness;
import info.textgrid.lab.collatex.process.EquivalenceSetTokenComparator;

import java.util.Comparator;

import org.eclipse.core.runtime.ListenerList;

import com.google.common.collect.Iterables;

import eu.interedition.collatex.Witness;
import eu.interedition.collatex.graph.VariantGraph;
import eu.interedition.collatex.matching.EqualityTokenComparator;

/**
 * Contains all data relevant to a running or finished collation.
 */
public class CollationResults {

	public interface ICollationResultsListener {

		/**
		 * Triggered when a different
		 * {@linkplain CollationResults#getFocusedWitness() witness has been
		 * focused}.
		 * 
		 * @param results
		 *            the {@link CollationResults} object in which the focus has
		 *            changed
		 */
		public void focusChanged(CollationResults results);
	}

	private final ListenerList listeners = new ListenerList();

	public void addListener(final ICollationResultsListener listener) {
		listeners.add(listener);
	}

	public void removeListener(final ICollationResultsListener listener) {
		listeners.remove(listener);
	}

	protected void notifyFocusChanged() {
		for (final Object listener : listeners.getListeners())
			((ICollationResultsListener) listener).focusChanged(this);
	}

	private CollationSet collationSet;
	private Equivalences equivalences;
	private VariantGraph variantGraph;
	private EqualityTokenComparator comparator;
	private BasicWitness focusedWitness;
	private CollationSetWitness focusedCSWitness;

	public CollationSet getCollationSet() {
		return collationSet;
	}

	public void setCollationSet(final CollationSet collationSet) {
		this.collationSet = collationSet;
	}

	public Equivalences getEquivalences() {
		return equivalences;
	}

	public void setEquivalences(final Equivalences equivalences) {
		this.equivalences = equivalences;
		this.comparator = null;
	}

	/**
	 * @return The variant graph that contains the actual results of the
	 *         collation.
	 */
	public VariantGraph getVariantGraph() {
		return variantGraph;
	}

	public void setVariantGraph(final VariantGraph variantGraph) {
		this.variantGraph = variantGraph;
	}

	/**
	 * Returns the {@link Comparator} that is used to compare tokens in this
	 * {@link CollationSet}.
	 */
	public EqualityTokenComparator getComparator() {
		if (comparator == null)
			if (equivalences == null)
				comparator = new EqualityTokenComparator();
			else
				comparator = new EquivalenceSetTokenComparator(equivalences);
		return comparator;
	}

	/**
	 * In user interfaces representing the CollationSet, a witness can be
	 * <em>focused</em>. While CollateX actual collation is baseless, this
	 * focused witness should be considered the base text by visualisations of
	 * the collation result that assume or highlight a base text.
	 * 
	 * <p>
	 * Clients can {@linkplain #addListener(ICollationResultsListener) register}
	 * a {@linkplain ICollationResultsListener} to be notified when the witness
	 * changes. Clients can alternatively use {@link #getFocusedWitness()} to
	 * retrieve the {@link Witness} from the {@linkplain #getVariantGraph()
	 * variant graph} corresponding to the focused witness.
	 */
	public CollationSetWitness getFocusedCSWitness() {
		if (focusedCSWitness == null || !focusedCSWitness.isActive())
			return Iterables.tryFind(getCollationSet().getWitnesses(),
					CollationSetWitness.IS_ACTIVE).or(
							getCollationSet().getWitnesses().get(0));
		else
			return focusedCSWitness;
	}

	/**
	 * Returns the {@link Witness} to be used as base text. See
	 * {@link #getFocusedCSWitness()} for an explanation of the concept.
	 */
	public BasicWitness getFocusedWitness() {
		if (focusedWitness == null && getFocusedCSWitness() != null
				&& variantGraph != null)
			for (final Witness witness : variantGraph.witnesses()) {
				final CollationSetWitness csWitness = ((BasicWitness) witness)
						.getCollationSetWitness();
				if (getFocusedCSWitness().equals(csWitness)) {
					focusedWitness = (BasicWitness) witness;
					break;
				}
			}

		return focusedWitness;

	}

	public void setFocusedCSWitness(final CollationSetWitness focusedCSWitness) {
		this.focusedCSWitness = focusedCSWitness;
		this.focusedWitness = null;
		notifyFocusChanged();
	}

}
