package info.textgrid.lab.collatex.model;

import info.textgrid.lab.core.model.TextGridObject;

import java.net.URI;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.eclipse.core.runtime.AssertionFailedException;
import org.eclipse.core.runtime.CoreException;

import com.google.common.base.Predicate;

/**
 * Reference to one witness in a collation set, including witness-specific
 * settings.
 */
@XmlRootElement(name = "witness")
public class CollationSetWitness {

	@Override
	public String toString() {
		return getSigil() + (active ? "" : Messages.CollationSetWitness_inactiveSuffix); //$NON-NLS-1$
	}

	@XmlTransient
	private URI uri;
	@XmlTransient
	private boolean active = true;

	@XmlAttribute(required = false)
	private String sigil;

	public static final Predicate<CollationSetWitness> IS_ACTIVE = new Predicate<CollationSetWitness>() {

		@Override
		public boolean apply(final CollationSetWitness input) {
			return input.isActive();
		}

	};

	public CollationSetWitness(final URI uri) {
		super();
		this.uri = uri;
	}

	public CollationSetWitness() {	}

	/**
	 * The URI of the object that contains the witness.
	 */
	@XmlAttribute
	public URI getUri() {
		return uri;
	}

	/**
	 * @see #getUri()
	 */
	public void setUri(final URI uri) {
		this.uri = uri;
	}

	/**
	 * Only active witnesses are considered when collating a collation set. The
	 * default is <code>true</code>.
	 */
	@XmlAttribute
	public boolean isActive() {
		return active;
	}

	public void setActive(final boolean active) {
		this.active = active;
	}

	/**
	 * Calculates a default value for the {@linkplain #getSigil() sigil} if no
	 * custom one {@linkplain #setSigil(String) has been set}. If possible, the
	 * {@linkplain TextGridObject#getTitle()} title of the object referenced by
	 * the {@linkplain #getUri() URI} will be used, otherwise the plain URI.
	 */
	@XmlTransient
	protected String getDefaultSigil() {
		try {
			final TextGridObject object = TextGridObject
					.getInstanceOffline(getUri());
			return object.getTitle();
		} catch (final AssertionFailedException e) {
			// No TextGrid URI? -> just return URI
		} catch (final CoreException e) {
			// Sth went wrong reading the title? -> just return URI
		}
		return getUri().toString();
	}

	/**
	 * A short string representation of this witness that will be used in
	 * visualisations of the result set etc. If no explicit sigil
	 * {@linkplain #setSigil(String) has been set}, a
	 * {@linkplain #getDefaultSigil() default representation} will be used.
	 */
	@XmlTransient
	public String getSigil() {
		if (sigil != null)
			return sigil;
		else
			return getDefaultSigil();
	}

	public void setSigil(final String sigil) {
		this.sigil = sigil;
	}

}
